@@ -1,15 +1,15 @@
 # Debian and Ubuntu system administration tools.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: October 25, 2016
+# Last Change: May 20, 2020
 # URL: https://debuntu-tools.readthedocs.io
 
 """
 Usage: debuntu-kernel-manager [OPTIONS] -- [APT_OPTIONS]
 
-Detect and remove old Linux kernel header and image packages that can be safely
-removed to conserve disk space and speed up apt-get runs that install or remove
-kernels.
+Detect and remove old Linux kernel header, image and modules packages that can
+be safely removed to conserve disk space and speed up apt-get runs that install
+or remove kernels.
 
 By default old packages are detected and reported on the command line but no
 changes are made. To actually remove old packages you need to use the -c,
@@ -36,6 +36,10 @@ Supported options:
     and --remove options will refuse to run apt-get and exit with an error
     instead. Use the -f or --force option to override this sanity check.
 
+  -p, --preserve-count=NUMBER
+
+    Preserve the NUMBER newest versions of the kernel packages (defaults to 2).
+
   -r, --remote-host=ALIAS
 
     Detect and remove old Linux kernel header and image packages on a remote
@@ -66,17 +70,37 @@ import coloredlogs
 from deb_pkg_tools.version import Version
 from executor import ExternalCommandFailed
 from executor.contexts import create_context
-from humanfriendly import AutomaticSpinner, Timer, compact, concatenate, pluralize
+from humanfriendly import Timer
 from humanfriendly.terminal import ansi_wrap, usage, warning
+from humanfriendly.terminal.spinners import AutomaticSpinner
+from humanfriendly.text import compact, concatenate, pluralize
 from property_manager import PropertyManager, cached_property, key_property, required_property
 from verboselogs import VerboseLogger
 
-REBOOT_REQUIRED_FILE = '/var/run/reboot-required'
+REBOOT_REQUIRED_FILE = "/var/run/reboot-required"
 """The absolute pathname of the file that exists when a system reboot is required (a string)."""
 
-REBOOT_REQUIRED_PACKAGES_FILE = '/var/run/reboot-required.pkgs'
+REBOOT_REQUIRED_PACKAGES_FILE = "/var/run/reboot-required.pkgs"
 """The absolute pathname of a file with details about why a system reboot is required (a string)."""
 
+# Compiled regular expression pattern to split package names
+# on dashes while ignoring dashes with a digit on both sides.
+PACKAGE_NAME_TOKENIZATION_PATTERN = re.compile(
+    r"""
+    # A dash with non-digits on both sides is fine.
+    (?<=\D) - (?=\D) |
+    # A dash with a non-digit on the left side is fine.
+    (?<=\D) - (?=\d) |
+    # A dash with a non-digit on the right side is fine.
+    (?<=\d) - (?=\D)
+""",
+    re.VERBOSE,
+)
+
+# Compiled regular expression pattern to match kernel versions encoded
+# in package names (assuming tokens are split on the pattern above).
+KERNEL_VERSION_PATTERN = re.compile(r"\d+\.\d+\.\d+-\d+")
+
 # Initialize a logger for this module.
 logger = VerboseLogger(__name__)
 
@@ -86,43 +110,42 @@ def main():
     # Initialize logging to the terminal and system log.
     coloredlogs.install(syslog=True)
     # Parse the command line arguments.
-    action = 'render_summary'
+    action = "render_summary"
     context_opts = dict()
     manager_opts = dict()
     try:
-        options, arguments = getopt.getopt(sys.argv[1:], 'cfr:vqh', [
-            'clean', 'remove',
-            'force', 'remote-host=',
-            'verbose', 'quiet', 'help',
-        ])
+        options, arguments = getopt.getopt(
+            sys.argv[1:],
+            "cfp:r:vqh",
+            ["clean", "remove", "force", "preserve-count=", "remote-host=", "verbose", "quiet", "help"],
+        )
         for option, value in options:
-            if option in ('-c', '--clean', '--remove'):
-                action = 'cleanup_packages'
-            elif option in ('-f', '--force'):
-                manager_opts['force'] = True
-            elif option in ('-r', '--remote-host'):
-                context_opts['ssh_alias'] = value
-            elif option in ('-v', '--verbose'):
+            if option in ("-c", "--clean", "--remove"):
+                action = "cleanup_packages"
+            elif option in ("-f", "--force"):
+                manager_opts["force"] = True
+            elif option in ("-p", "--preserve-count"):
+                manager_opts["preserve_count"] = int(value)
+            elif option in ("-r", "--remote-host"):
+                context_opts["ssh_alias"] = value
+            elif option in ("-v", "--verbose"):
                 coloredlogs.increase_verbosity()
-            elif option in ('-q', '--quiet'):
+            elif option in ("-q", "--quiet"):
                 coloredlogs.decrease_verbosity()
-            elif option in ('-h', '--help'):
+            elif option in ("-h", "--help"):
                 usage(__doc__)
                 return
             else:
                 raise Exception("Unhandled option!")
         # Any positional arguments are passed to apt-get.
-        manager_opts['apt_options'] = arguments
+        manager_opts["apt_options"] = arguments
     except Exception as e:
         warning("Failed to parse command line arguments! (%s)", e)
         sys.exit(1)
     # Execute the requested action(s).
     context = create_context(**context_opts)
     try:
-        manager = KernelPackageManager(
-            context=context,
-            **manager_opts
-        )
+        manager = KernelPackageManager(context=context, **manager_opts)
         getattr(manager, action)()
     except (CleanupError, ExternalCommandFailed) as e:
         logger.error("%s", e)
@@ -155,6 +178,16 @@ class KernelPackageManager(PropertyManager):
         """The number of kernel packages to preserve (an integer, defaults to 2)."""
         return 2
 
+    @property
+    def dry_run(self):
+        """:data:`True` if :attr:`cleanup_command` performs a dry run, :data:`False` otherwise."""
+        return any(
+            re.match("^-[^-]", argument)
+            and "s" in argument
+            or argument in ("--simulate", "--just-print", "--dry-run", "--recon", "--no-act")
+            for argument in self.apt_options
+        )
+
     @cached_property
     def installed_packages(self):
         """
@@ -164,7 +197,7 @@ class KernelPackageManager(PropertyManager):
         ``dpkg --list`` command.
         """
         mapping = {}
-        output = self.context.capture('dpkg', '--list')
+        output = self.context.capture("dpkg", "--list")
         for line in output.splitlines():
             tokens = line.split()
             if len(tokens) >= 3:
@@ -176,16 +209,38 @@ class KernelPackageManager(PropertyManager):
     @cached_property
     def installed_header_packages(self):
         """A list of :class:`MaybeKernelPackage` objects for the installed Linux kernel header packages."""
-        return sorted((package for package in self.installed_packages.values()
-                       if package.is_installed and package.is_header_package),
-                      key=lambda package: (package.version, package.name))
+        return sorted(
+            (
+                package
+                for package in self.installed_packages.values()
+                if package.is_installed and package.is_header_package
+            ),
+            key=lambda package: (package.version, package.name),
+        )
 
     @cached_property
     def installed_kernel_packages(self):
         """A list of :class:`MaybeKernelPackage` objects for the installed kernel images."""
-        return sorted((package for package in self.installed_packages.values()
-                       if package.is_installed and package.is_kernel_package),
-                      key=lambda package: (package.version, package.name))
+        return sorted(
+            (
+                package
+                for package in self.installed_packages.values()
+                if package.is_installed and package.is_kernel_package
+            ),
+            key=lambda package: (package.version, package.name),
+        )
+
+    @cached_property
+    def installed_modules_packages(self):
+        """A list of :class:`MaybeKernelPackage` objects for the installed kernel modules packages."""
+        return sorted(
+            (
+                package
+                for package in self.installed_packages.values()
+                if package.is_installed and package.is_modules_package
+            ),
+            key=lambda package: (package.version, package.name),
+        )
 
     @cached_property
     def installed_header_meta_packages(self):
@@ -193,9 +248,14 @@ class KernelPackageManager(PropertyManager):
         # We change the sort key for meta packages from (name, version) to just
         # the version so that the meta packages are listed in version order
         # despite the names being different.
-        return sorted((package for package in self.installed_packages.values()
-                       if package.is_installed and package.is_header_meta_package),
-                      key=lambda p: p.version)
+        return sorted(
+            (
+                package
+                for package in self.installed_packages.values()
+                if package.is_installed and package.is_header_meta_package
+            ),
+            key=lambda p: p.version,
+        )
 
     @cached_property
     def installed_image_meta_packages(self):
@@ -203,23 +263,38 @@ class KernelPackageManager(PropertyManager):
         # We change the sort key for meta packages from (name, version) to just
         # the version so that the meta packages are listed in version order
         # despite the names being different.
-        return sorted((package for package in self.installed_packages.values()
-                       if package.is_installed and package.is_image_meta_package),
-                      key=lambda p: p.version)
+        return sorted(
+            (
+                package
+                for package in self.installed_packages.values()
+                if package.is_installed and package.is_image_meta_package
+            ),
+            key=lambda p: p.version,
+        )
 
     @cached_property
     def installed_package_groups(self):
         """A list of sets with :class:`MaybeKernelPackage` objects for installed header and kernel packages."""
         grouped_packages = collections.defaultdict(list)
         for package in self.installed_packages.values():
-            if package.is_installed and package.is_kernel_or_header_package:
+            if package.is_installed and package.is_supported_package:
                 grouped_packages[package.version_in_name].append(package)
         return sorted(grouped_packages.values(), key=lambda group: group[0].version)
 
+    @cached_property
+    def active_kernel_release(self):
+        """The output of ``uname --kernel-release`` (a string)."""
+        return self.context.capture("uname", "--kernel-release")
+
     @cached_property
     def active_kernel_package(self):
         """The package name for the running kernel (a string)."""
-        return 'linux-image-%s' % self.context.capture('uname', '--kernel-release')
+        return "linux-image-%s" % self.active_kernel_release
+
+    @cached_property
+    def reboot_required(self):
+        """:data:`True` if :data:`REBOOT_REQUIRED_FILE` exists, :data:`False` otherwise."""
+        return self.context.exists(REBOOT_REQUIRED_FILE)
 
     @cached_property
     def removable_package_groups(self):
@@ -244,9 +319,11 @@ class KernelPackageManager(PropertyManager):
             else:
                 # Consider removing nonactive package groups with a kernel image.
                 may_remove.append(group)
-        # Prefer new kernels over old kernels and respect
-        # the configured number of packages to preserve.
-        return will_remove + may_remove[:-preserve_count]
+        if preserve_count > 0:
+            # Prefer new kernels over old kernels and respect
+            # the configured number of packages to preserve.
+            may_remove = may_remove[:-preserve_count]
+        return will_remove + may_remove
 
     @cached_property
     def removable_header_packages(self):
@@ -258,34 +335,30 @@ class KernelPackageManager(PropertyManager):
         """A list of :class:`MaybeKernelPackage` objects for kernel packages considered to be removable."""
         return sorted(pkg for grp in self.removable_package_groups for pkg in grp if pkg.is_kernel_package)
 
+    @cached_property
+    def removable_modules_packages(self):
+        """A list of :class:`MaybeKernelPackage` objects for modules packages considered to be removable."""
+        return sorted(pkg for grp in self.removable_package_groups for pkg in grp if pkg.is_modules_package)
+
     @cached_property
     def removable_packages(self):
         """A list of :class:`MaybeKernelPackage` objects for kernel related packages considered to be removable."""
-        return self.removable_header_packages + self.removable_kernel_packages
+        return self.removable_header_packages + self.removable_kernel_packages + self.removable_modules_packages
 
     @cached_property
     def running_newest_kernel(self):
         """:data:`True` if the newest kernel is currently active, :data:`False` otherwise."""
         return any(package.name == self.active_kernel_package for package in self.installed_package_groups[-1])
 
-    @cached_property
-    def dry_run(self):
-        """:data:`True` if :attr:`cleanup_command` performs a dry run, :data:`False` otherwise."""
-        return any(
-            re.match('^-[^-]', argument) and 's' in argument or
-            argument in ('--simulate', '--just-print', '--dry-run', '--recon', '--no-act')
-            for argument in self.apt_options
-        )
-
     @cached_property
     def cleanup_command(self):
         """A list of strings with the ``apt-get`` command to remove old packages."""
         command_line = []
         if self.removable_packages:
-            command_line.extend(['apt-get', 'remove', '--purge'])
+            command_line.extend(["apt-get", "remove", "--purge"])
             command_line.extend(self.apt_options)
             for group in self.removable_package_groups:
-                command_line.extend(package.name for package in group)
+                command_line.extend(sorted(package.name for package in group))
         return command_line
 
     def render_summary(self):
@@ -294,34 +367,49 @@ class KernelPackageManager(PropertyManager):
         with AutomaticSpinner(label="Gathering information about %s" % self.context):
             # Report the installed Linux kernel image meta package(s).
             if self.installed_image_meta_packages:
-                logger.info("Found %s installed:",
-                            pluralize(len(self.installed_image_meta_packages),
-                                      "Linux kernel image meta package"))
+                logger.info(
+                    "Found %s installed:",
+                    pluralize(len(self.installed_image_meta_packages), "Linux kernel image meta package"),
+                )
                 for package in self.installed_image_meta_packages:
                     logger.info(" - %s (%s)", package.name, package.version)
                 if len(self.installed_image_meta_packages) > 1:
                     names = concatenate(pkg.name for pkg in self.installed_image_meta_packages)
-                    logger.warning(compact("""
-                        You have more than one Linux kernel image meta package
-                        installed ({names}) which means automatic package
-                        removal can be unreliable!
-                    """, names=names))
-                    logger.verbose(compact("""
-                        I would suggest to stick to one Linux kernel
-                        image meta package, preferably the one that
-                        matches the newest kernel :-)
-                    """))
+                    logger.warning(
+                        compact(
+                            """
+                            You have more than one Linux kernel image meta
+                            package installed ({names}) which means automatic
+                            package removal can be unreliable!
+                            """,
+                            names=names,
+                        )
+                    )
+                    logger.verbose(
+                        compact(
+                            """
+                            I would suggest to stick to one Linux kernel image
+                            meta package, preferably the one that matches the
+                            newest kernel :-)
+                            """
+                        )
+                    )
             else:
-                logger.warning(compact("""
-                    It looks like there's no Linux kernel image meta package
-                    installed! I hope you've thought about how to handle
-                    security updates?
-                """))
+                logger.warning(
+                    compact(
+                        """
+                        It looks like there's no Linux kernel image meta
+                        package installed! I hope you've thought about how to
+                        handle security updates?
+                        """
+                    )
+                )
             # Report the installed Linux kernel header/image package(s).
             logger.verbose("Checking for removable packages on %s ..", self.context)
             package_types = (
                 (self.installed_kernel_packages, "image", True),
                 (self.installed_header_packages, "header", False),
+                (self.installed_modules_packages, "modules", False),
             )
             for collection, label, expected in package_types:
                 if collection:
@@ -329,28 +417,27 @@ class KernelPackageManager(PropertyManager):
                     for group in self.installed_package_groups:
                         matching_packages = sorted(package.name for package in group if package in collection)
                         active_group = any(package.name == self.active_kernel_package for package in group)
-                        removable_group = (group in self.removable_package_groups)
+                        removable_group = group in self.removable_package_groups
                         if matching_packages:
                             logger.info(
                                 " - %s (%s)",
                                 concatenate(matching_packages),
-                                ansi_wrap("removable", color='green')
-                                if removable_group else ansi_wrap(
-                                    "the active kernel" if active_group else
-                                    ("one of %i newest kernels" % self.preserve_count),
-                                    color='blue'
+                                ansi_wrap("removable", color="green")
+                                if removable_group
+                                else ansi_wrap(
+                                    "the active kernel"
+                                    if active_group
+                                    else ("one of %i newest kernels" % self.preserve_count),
+                                    color="blue",
                                 ),
                             )
                 elif expected:
                     logger.warning("No installed %s packages found, this can't be right?!", label)
             # Report the removable packages.
             if self.removable_packages:
-                logger.info(
-                    "Found %s that can be removed.",
-                    pluralize(len(self.removable_packages), "package"),
-                )
+                logger.info("Found %s that can be removed.", pluralize(len(self.removable_packages), "package"))
                 # Report the shell command to remove the packages.
-                logger.verbose("Command to remove packages: %s", ' '.join(self.cleanup_command))
+                logger.verbose("Command to remove packages: %s", " ".join(self.cleanup_command))
             else:
                 logger.info("No packages need to be removed! :-)")
 
@@ -359,8 +446,8 @@ class KernelPackageManager(PropertyManager):
         Run ``apt-get`` to cleanup removable kernel related packages.
 
         :param options: Any keyword arguments are passed on to the
-                        :func:`~executor.contexts.AbstractContext.execute()`
-                        method of the :class:`context` object.
+                        :meth:`~executor.contexts.AbstractContext.execute()`
+                        method of the :attr:`context` object.
         :returns: :data:`True` if a system reboot is required (to switch to the
                   newest installed kernel image or because security updates
                   have been installed), :data:`False` otherwise.
@@ -371,32 +458,38 @@ class KernelPackageManager(PropertyManager):
         self.render_summary()
         if self.cleanup_command:
             if len(self.installed_image_meta_packages) > 1 and not self.force:
-                raise CleanupError(compact("""
-                    Refusing to cleanup kernel related packages on {system}
-                    because results can be unreliable when multiple Linux
-                    kernel image meta packages are installed! You can use
-                    the -f, --force option to override this sanity check.
-                """, system=self.context))
+                raise CleanupError(
+                    compact(
+                        """
+                        Refusing to cleanup kernel related packages on {system}
+                        because results can be unreliable when multiple Linux
+                        kernel image meta packages are installed! You can use
+                        the -f, --force option to override this sanity check.
+                        """,
+                        system=self.context,
+                    )
+                )
             # Check if the packaging system has signaled that a system reboot
             # is required before we run the `apt-get remove' command.
-            reboot_required_before = self.context.exists(REBOOT_REQUIRED_FILE)
+            reboot_required_before = self.reboot_required
             # Get the set of installed packages before we run `apt-get remove'.
             installed_packages_before = set(p for p in self.installed_packages.values() if p.is_installed)
             # Actually run the `apt-get remove' command.
             logger.info("Removing %s on %s ..", pluralize(len(self.removable_packages), "package"), self.context)
             self.context.execute(*self.cleanup_command, sudo=True, **options)
             # Make sure `/etc/apt/apt.conf.d/01autoremove-kernels' is up to date.
-            auto_removal_script = '/etc/kernel/postinst.d/apt-auto-removal'
+            auto_removal_script = "/etc/kernel/postinst.d/apt-auto-removal"
             logger.verbose("Checking if %s needs to be run ..", auto_removal_script)
-            if self.context.test('test', '-x', auto_removal_script):
+            if self.context.test("test", "-x", auto_removal_script):
                 if self.dry_run:
                     logger.verbose("Skipping %s script because we're performing a dry-run.", auto_removal_script)
                 else:
                     logger.verbose("Running %s script ..", auto_removal_script)
-                    active_kernel = self.context.capture('uname', '--kernel-release')
-                    if not self.context.execute(auto_removal_script, active_kernel, check=False, sudo=True):
-                        logger.warning("Failed to update auto-remove statuses! (%s reported an error)",
-                                       auto_removal_script)
+                    auto_removal_command = [auto_removal_script, self.active_kernel_release]
+                    if not self.context.execute(*auto_removal_command, check=False, sudo=True, tty=False):
+                        logger.warning(
+                            "Failed to update auto-remove statuses! (%s reported an error)", auto_removal_script
+                        )
             logger.info("Done! (took %s)", timer)
             # The `apt-get remove' command invalidates all of our cached data
             # so we need to refresh our cached properties to avoid stale data.
@@ -415,17 +508,13 @@ class KernelPackageManager(PropertyManager):
                     else:
                         logger.info("System reboot is avoidable! Removing signal file(s) ..")
                         self.context.execute(
-                            'rm', '--force',
-                            REBOOT_REQUIRED_FILE,
-                            REBOOT_REQUIRED_PACKAGES_FILE,
-                            sudo=True,
+                            "rm", "--force", REBOOT_REQUIRED_FILE, REBOOT_REQUIRED_PACKAGES_FILE, sudo=True, tty=False
                         )
-
         # Inform the operator and caller about whether a reboot is required.
         if not self.running_newest_kernel:
             logger.info("System reboot needed (not yet running the newest kernel).")
             return True
-        elif self.context.exists(REBOOT_REQUIRED_FILE):
+        elif self.reboot_required:
             logger.info("System reboot needed (%s exists).", REBOOT_REQUIRED_FILE)
             return True
         else:
@@ -438,7 +527,7 @@ class MaybeKernelPackage(PropertyManager):
     """Dumb container for entries parsed from ``dpkg --list`` output."""
 
     # Explicitly define the sort order of the key properties.
-    key_properties = 'name', 'version'
+    key_properties = "name", "version"
 
     @key_property
     def name(self):
@@ -455,71 +544,157 @@ class MaybeKernelPackage(PropertyManager):
     @property
     def is_installed(self):
         """:data:`True` if the package is installed (or configuration files remain), :data:`False` otherwise."""
-        return self.status in ('ii', 'rc')
+        return self.status in ("ii", "rc")
+
+    @cached_property
+    def tokenized_name(self):
+        """
+        The tokenized name of the package (a list of strings).
+
+        The value of this property is computed by calling
+        :func:`tokenize_package_name()` on :attr:`name`.
+        """
+        return tokenize_package_name(self.name)
 
     @cached_property
     def is_header_meta_package(self):
         """:data:`True` if the package is a Linux kernel header meta package, :data:`False` otherwise."""
-        tokens = self.name.split('-')
-        return (len(tokens) >= 3 and
-                tokens[0] == 'linux' and
-                'headers' in tokens and
-                not any(t[0].isdigit() for t in tokens))
+        return (
+            len(self.tokenized_name) >= 3
+            and self.tokenized_name[0] == "linux"
+            and "headers" in self.tokenized_name
+            and not any(map(is_kernel_version, self.tokenized_name))
+        )
 
     @cached_property
     def is_image_meta_package(self):
         """:data:`True` if the package is a Linux kernel image meta package, :data:`False` otherwise."""
-        tokens = self.name.split('-')
-        return (len(tokens) >= 3 and
-                tokens[0] == 'linux' and
-                'image' in tokens and
-                not any(t[0].isdigit() for t in tokens))
+        return (
+            len(self.tokenized_name) >= 3
+            and self.tokenized_name[0] == "linux"
+            and "image" in self.tokenized_name
+            and not any(map(is_kernel_version, self.tokenized_name))
+        )
 
     @cached_property
     def is_header_package(self):
         """:data:`True` if the package is a specific version of the Linux kernel headers, :data:`False` otherwise."""
-        tokens = self.name.split('-')
-        return (len(tokens) >= 3 and
-                tokens[0] == 'linux' and
-                tokens[1] == 'headers' and
-                tokens[2][0].isdigit())
+        return (
+            # linux-headers-$VERSION-generic
+            len(self.tokenized_name) >= 3
+            and self.tokenized_name[0] == "linux"
+            and self.tokenized_name[1] == "headers"
+            and is_kernel_version(self.tokenized_name[2])
+        ) or (
+            # linux-raspi2-headers-$VERSION
+            len(self.tokenized_name) >= 4
+            and self.tokenized_name[0] == "linux"
+            and self.tokenized_name[1].isalnum()
+            and self.tokenized_name[2] == "headers"
+            and is_kernel_version(self.tokenized_name[3])
+        )
 
     @cached_property
     def is_kernel_package(self):
         """:data:`True` if the package is a specific version of the Linux kernel image, :data:`False` otherwise."""
-        tokens = self.name.split('-')
-        return (tokens and tokens[0] == 'linux' and
-                ((len(tokens) >= 3 and tokens[1] == 'image' and tokens[2][0].isdigit()) or
-                 (len(tokens) >= 4 and tokens[1] == 'image' and tokens[2] == 'extra' and tokens[3][0].isdigit()) or
-                 (len(tokens) >= 4 and tokens[1] == 'signed' and tokens[2] == 'image' and tokens[3][0].isdigit())))
+        return (
+            (
+                # linux-image-$VERSION
+                len(self.tokenized_name) >= 3
+                and self.tokenized_name[0] == "linux"
+                and self.tokenized_name[1] == "image"
+                and is_kernel_version(self.tokenized_name[2])
+            )
+            or (
+                # linux-image-extra-$VERSION
+                len(self.tokenized_name) >= 4
+                and self.tokenized_name[0] == "linux"
+                and self.tokenized_name[1] == "image"
+                and self.tokenized_name[2] == "extra"
+                and is_kernel_version(self.tokenized_name[3])
+            )
+            or (
+                # linux-signed-image-$VERSION
+                len(self.tokenized_name) >= 4
+                and self.tokenized_name[0] == "linux"
+                and self.tokenized_name[1] == "signed"
+                and self.tokenized_name[2] == "image"
+                and is_kernel_version(self.tokenized_name[3])
+            )
+        )
 
     @cached_property
-    def is_kernel_or_header_package(self):
-        """:data:`True` if the package is a Linux kernel image or headers package, :data:`False` otherwise."""
-        return self.is_header_package or self.is_kernel_package
+    def is_modules_package(self):
+        """:data:`True` if the package contains Linux kernel modules, :data:`False` otherwise."""
+        return (
+            # linux-modules-$VERSION-$VARIANT
+            len(self.tokenized_name) >= 3
+            and self.tokenized_name[0] == "linux"
+            and self.tokenized_name[1] == "modules"
+            and is_kernel_version(self.tokenized_name[2])
+        ) or (
+            # linux-modules-extra-$VERSION-$VARIANT
+            len(self.tokenized_name) >= 4
+            and self.tokenized_name[0] == "linux"
+            and self.tokenized_name[1] == "modules"
+            and self.tokenized_name[2] == "extra"
+            and is_kernel_version(self.tokenized_name[3])
+        )
+
+    @cached_property
+    def is_supported_package(self):
+        """:data:`True` if the package concerns a Linux kernel image or modules or headers, :data:`False` otherwise."""
+        return self.is_header_package or self.is_kernel_package or self.is_modules_package
 
     @cached_property
     def version_in_name(self):
         """The version encoded in the name of the package (a string or :data:`None`)."""
-        if self.is_kernel_or_header_package:
-            tokens = self.name.split('-')
-            # Remove the static prefix from the package name.
-            for prefix in 'linux', 'signed', 'image', 'extra', 'headers':
-                if tokens and tokens[0] == prefix:
-                    tokens.pop(0)
-            # Ignore the kernel type that may be encoded in the package name.
-            if tokens[-1].isalpha():
-                tokens.pop(-1)
-            # Reconstruct the version number encoded in the name of the package.
-            return '-'.join(tokens)
+        if self.is_supported_package:
+            for token in self.tokenized_name:
+                if is_kernel_version(token):
+                    return token
 
     @cached_property
     def kernel_type(self):
         """The kernel type encoded in the name of the package (a string or :data:`None`)."""
-        if self.is_kernel_or_header_package:
-            tokens = self.name.split('-')
-            if tokens[-1].isalpha():
-                return tokens[-1]
+        if self.is_supported_package:
+            if self.tokenized_name[-1].isalpha():
+                return self.tokenized_name[-1]
+
+
+def tokenize_package_name(package_name):
+    """
+    Split a (kernel) package name into tokens.
+
+    :param package_name: The name of a package (a string).
+    :returns: A list of strings.
+
+    The :func:`tokenize_package_name()` function uses a regular expression to
+    tokenize package names by splitting on dashes while ignoring dashes with a
+    digit on both sides (i.e. dashes that are part of a version number). For
+    example:
+
+    >>> from debuntu_tools.kernel_manager import tokenize_package_name
+    >>> tokenize_package_name('linux-image-4.4.0-72-generic')
+    ['linux', 'image', '4.4.0-72', 'generic']
+    """
+    return PACKAGE_NAME_TOKENIZATION_PATTERN.split(package_name)
+
+
+def is_kernel_version(token):
+    """
+    Check whether a token in a package name looks like a kernel version.
+
+    :param token: The token to check (a string).
+    :returns: :data:`True` if the token looks like a kernel version,
+              :data:`False` otherwise.
+
+    The :func:`is_kernel_version()` function uses a regular expression
+    to check whether the given string looks like a kernel version of
+    the form ``4.4.0-72``. The string is assumed to be a package
+    name token returned by :func:`tokenize_package_name()`.
+    """
+    return KERNEL_VERSION_PATTERN.match(token) is not None
 
 
 class CleanupError(Exception):
